@@ -23,7 +23,6 @@ int exit_fn(int sd, int argc, char **argv);
 char *strtoke(char *str, char *ws, char *delim);
 void exit_func(void);
 void sigexit(int sig);
-void sigalarm(int sig);
 
 typedef int (*cmd_fn_t)(int, int, char **);
 
@@ -68,10 +67,8 @@ int user_fn(int sd, int argc, char **argv) {
   char fl_name = 0, fl_uid = 0, fl_vid = 0, fl_pid = 0;
   int c, ret;
 
-#ifdef sun
   extern char* optarg;
   extern int optind;
-#endif
 
   optind = 0;
   while ((c = getopt(argc, argv, "n:i:v:p:h")) != -1) {
@@ -101,8 +98,6 @@ int user_fn(int sd, int argc, char **argv) {
     }
   }
 
-  alarm(TICKLE_INTERVAL);
-
   ret = dlp_ReadUserInfo(sd, &U);
   if (ret < 0) {
     printf("dlp_ReadUserInfo: err %d\n", ret);
@@ -128,8 +123,6 @@ int user_fn(int sd, int argc, char **argv) {
   U.succSyncDate = time(NULL);
   U.lastSyncDate = U.succSyncDate;
 
-  alarm(TICKLE_INTERVAL);
-
   ret = dlp_WriteUserInfo(sd, &U);
   if (ret < 0) {
     printf("dlp_WriteUserInfo: err %d\n", ret);
@@ -193,8 +186,6 @@ ls_fn(int sd, int argc, char **argv)
     struct DBInfo info;
     long tag;
 
-    alarm(TICKLE_INTERVAL);
-
     /*
      * The databases are numbered starting at 0.  The first 12 are in
      * ROM, and the rest are in RAM.  The high two bits of the flags
@@ -301,11 +292,6 @@ void handle_user_commands(int sd) {
 
 int main(int argc, char **argv) {
   struct pi_sockaddr addr;
-#ifdef HAVE_SIGACTION
-  struct sigaction sigact;
-#else
-  struct sigvec vec;
-#endif
   
   int sd;
   int ret;
@@ -323,11 +309,10 @@ int main(int argc, char **argv) {
     exit(1);
   }
 
-  addr.sa_family = PI_AF_SLP;
-  addr.port = 3;
-  strcpy(addr.device, argv[1]);
+  addr.pi_family = PI_AF_SLP;
+  strcpy(addr.pi_device, argv[1]);
 
-  ret = pi_bind(sd, &addr, sizeof(addr));
+  ret = pi_bind(sd, (struct sockaddr*)&addr, sizeof(addr));
   if (ret == -1) {
     perror("pi_bind");
     exit(1);
@@ -341,8 +326,8 @@ int main(int argc, char **argv) {
 
   printf("Connecting to Pilot at %s...", argv[1]); fflush(stdout);
 
-  ret = pi_accept(sd, &addr, NULL);
-  if (ret == -1) {
+  sd = pi_accept(sd, 0, NULL);
+  if (sd == -1) {
     perror("pi_accept");
     exit(1);
   }
@@ -350,31 +335,7 @@ int main(int argc, char **argv) {
   printf("connected.\n");
 
   /* Stayin' alive, stayin' alive... */
-  ticklish_pi_socket = find_pi_socket(sd);
-
-  /* Set up signal handlers. */
-
-#ifdef HAVE_SIGACTION
-  sigact.sa_handler = sigalarm;
-  sigemptyset(&sigact.sa_mask);
-  sigact.sa_flags = 0;
-  sigaction(SIGALRM, &sigact, NULL);
-
-  sigact.sa_handler = sigexit;
-  sigaction(SIGINT, &sigact, NULL);
-  sigaction(SIGTERM, &sigact, NULL);
-#else
-  vec.sv_handler = sigalarm;
-  vec.sv_mask = 0;
-  vec.sv_onstack = 0;
-  sigvec(SIGALRM, &vec, NULL);
-  
-  vec.sv_handler = sigexit;
-  sigvec(SIGINT, &vec, NULL);
-  sigvec(SIGTERM, &vec, NULL);
-#endif
-
-  alarm(TICKLE_INTERVAL);
+  pi_watchdog(sd, TICKLE_INTERVAL);
 
   socket_descriptor = sd;
   atexit(exit_func);
@@ -383,16 +344,6 @@ int main(int argc, char **argv) {
   exit(0);
 }
 
-void sigalarm(int sig) {
-  struct padp pd;
-  pd.type = padTickle;
-  pd.flags = 0x00;
-  pd.size = 0x00;
-  padp_tx(ticklish_pi_socket, (void *)&pd, 0, padTickle);
-  pi_socket_flush(ticklish_pi_socket);
-  alarm(TICKLE_INTERVAL);
-}
-
 void sigexit(int sig) {
   exit(0);
 }
@@ -401,7 +352,6 @@ void exit_func(void) {
 #ifdef DEBUG
   fprintf(stderr, "\n\n================== EXITING ===================\n\n\n");
 #endif
-  alarm(0);
   dlp_EndOfSync(socket_descriptor, 0);
   pi_close(socket_descriptor);
 }
