@@ -25,6 +25,7 @@ int main(int argc, char *argv[])
   struct PilotUser U;
   struct SysInfo S;
   struct CardInfo C;
+  struct NetSyncInfo N;
   unsigned long romversion;
   int ret;
 
@@ -38,11 +39,10 @@ int main(int argc, char *argv[])
     exit(1);
   }
     
-  addr.sa_family = PI_AF_SLP;
-  addr.port = 3;
-  strcpy(addr.device,argv[1]);
+  addr.pi_family = PI_AF_SLP;
+  strcpy(addr.pi_device,argv[1]);
   
-  ret = pi_bind(sd, &addr, sizeof(addr));
+  ret = pi_bind(sd, (struct sockaddr*)&addr, sizeof(addr));
   if(ret == -1) {
     perror("pi_bind");
     exit(1);
@@ -97,6 +97,11 @@ int main(int argc, char *argv[])
   
   printf( "ROM Version through ReadFeature: 0x%8.8lX\n", romversion);
   
+  if (dlp_ReadNetSyncInfo(sd, &N) >= 0) {
+    printf( "NetSync: LAN sync = %d, Host name = '%s', address = '%s', netmask ='%s'\n",
+    	N.lansync, N.PCName, N.PCAddr, N.PCMask);
+  }
+  
   pi_close(sd);
   exit(0);
 }
