@@ -20,6 +20,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include "pi-source.h"
 #include "pi-socket.h"
 #include "pi-mail.h"
@@ -28,8 +29,8 @@
 
 #include <sys/types.h>
 #include <sys/socket.h>
-#include <arpa/inet.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <netdb.h>
 
 extern time_t parsedate(char * p);
@@ -251,10 +252,9 @@ int main(int argc, char *argv[])
   "";
 #endif
 
-#ifdef sun
   extern char* optarg;
   extern int optind;
-#endif
+  
 
   if (getenv("SENDMAIL"))
     sendmail = getenv("SENDMAIL");
@@ -274,9 +274,10 @@ int main(int argc, char *argv[])
     port = getenv("PILOTPORT");
     
   signal(SIGINT, sigint);
-  
+
   optind = 0;
-  while ((c = getopt(argc, argv, "s:p:d:f:h:u:p:h:P:k:")) != -1) {
+  
+  while ((c = getopt(argc, argv, "s:p:d:f:h:u:p:h:P:k:")) != EOF) {
     switch (c) {
       case 's':
         sendmail = optarg;
@@ -306,6 +307,8 @@ int main(int argc, char *argv[])
         Help(argv);
     }
   }
+  argc -= optind;
+  argv += optind;
   
   if (!strlen(port)) {
     fprintf(stderr, "Port must be set.\n\n");
@@ -327,6 +330,11 @@ int main(int argc, char *argv[])
       Help(argv);
     } 
   }
+
+  if ( !strlen( pop_pass ) )
+    {
+      pop_pass = getpass( "POP password: " );
+    }
   
   if (strlen(sendmail)) {
     if (!strlen(dispose) || (strcmp(dispose,"keep") &&
@@ -344,11 +352,10 @@ int main(int argc, char *argv[])
   
   lost = dupe = rec = sent = 0;
   
-  addr.sa_family = PI_AF_SLP;
-  addr.port = 3;
-  strcpy(addr.device, port);
+  addr.pi_family = PI_AF_SLP;
+  strcpy(addr.pi_device, port);
   
-  ret = pi_bind(sd, &addr, sizeof(addr));
+  ret = pi_bind(sd, (struct sockaddr*)&addr, sizeof(addr));
   if(ret == -1) {
     perror("pi_bind");
     exit(1);
@@ -397,12 +404,12 @@ int main(int argc, char *argv[])
   p.filtersubject = 0;
   
   if (pi_version(sd) > 0x0100) {
-    if (dlp_ReadAppPreference(sd, db, makelong("mail"), 1, 1, 0xffff, buffer, 0, 0)>=0) {
+    if (dlp_ReadAppPreference(sd, makelong("mail"), 1, 1, 0xffff, buffer, 0, 0)>=0) {
       printf("Got local backup mail preferences\n"); /* 2 for remote prefs */
       unpack_MailPrefs(&p, buffer, 0);
     } else {
       printf("Unable to get mail preferences, trying current\n");
-      if (dlp_ReadAppPreference(sd, db, makelong("mail"), 1, 1, 0xffff, buffer, 0, 0)>=0) {
+      if (dlp_ReadAppPreference(sd, makelong("mail"), 1, 1, 0xffff, buffer, 0, 0)>=0) {
         printf("Got local current mail preferences\n"); /* 2 for remote prefs */
         unpack_MailPrefs(&p, buffer, 0);
       } else
@@ -412,7 +419,7 @@ int main(int argc, char *argv[])
     if (tai.signature)
       free(tai.signature);
       
-    if (dlp_ReadAppPreference(sd, db, makelong("mail"), 3, 1, 0xffff, buffer, 0, 0)>0) {
+    if (dlp_ReadAppPreference(sd, makelong("mail"), 3, 1, 0xffff, buffer, 0, 0)>0) {
       tai.signature=strdup(buffer);
     }
 
@@ -682,7 +689,7 @@ int main(int argc, char *argv[])
     
     if (strlen(msg) > p.truncate) {
       /* We could truncate it, but we won't for now */
-      fprintf(stderr, "Message %d too large (%d bytes)\n", i, strlen(msg));
+      fprintf(stderr, "Message %d too large (%ld bytes)\n", i, (long)strlen(msg));
       free_Mail(&t);
       continue;
     }
