@@ -56,11 +56,10 @@ void Connect(void) {
     exit(1);
   }
 
-  addr.sa_family = PI_AF_SLP;
-  addr.port = 3;
-  strcpy(addr.device,device);
+  addr.pi_family = PI_AF_SLP;
+  strcpy(addr.pi_device,device);
   
-  ret = pi_bind(sd, &addr, sizeof(addr));
+  ret = pi_bind(sd, (struct sockaddr*)&addr, sizeof(addr));
   if(ret == -1) {
     perror("pi_bind");
     exit(1);
@@ -109,6 +108,7 @@ void VoidSyncFlags(void)
                                   any Desktop software that databases
                                   have been altered, and that a slow
                                   sync is necessary */
+    U.lastSyncDate = U.succSyncDate = time(0);
     dlp_WriteUserInfo(sd, &U);
   } 
 }
@@ -189,7 +189,10 @@ void Fetch(char * dbname)
     exit(1);
   }
   	
-  dlp_FindDBInfo(sd, 0, 0, dbname, 0, 0, &info);
+  if (dlp_FindDBInfo(sd, 0, 0, dbname, 0, 0, &info)<0) {
+    printf("Unable to locate database '%s', fetch skipped.\n", dbname);
+    return;
+  }
         
   strcpy(name, dbname);
   if (info.flags & dlpDBFlagResource)
@@ -403,6 +406,37 @@ void Install(char * filename)
   printf("Install done\n");
 }
 
+void Merge(char * filename)
+{
+  struct pi_file * f;
+  
+  Connect();
+
+  if ( dlp_OpenConduit(sd) < 0) {
+    puts("Exiting on cancel. All data not restored.");
+    exit(1);
+  }
+
+  f = pi_file_open(filename);
+  if (f==0) {
+    printf("Unable to open '%s'!\n", filename);
+    return;
+  }
+  
+  
+  printf("Merging %s... ", filename);
+  fflush(stdout);
+  if(pi_file_merge(f, sd, 0)<0)
+    printf("failed.\n");
+  else
+    printf("OK\n");
+  pi_file_close(f);
+  
+  VoidSyncFlags();
+
+  printf("Merge done\n");
+}
+
 void List(void)
 {
   struct DBInfo info;
@@ -480,6 +514,7 @@ void Help(void)
       printf("Where a command is one of: -b(ackup)  backupdir\n");
       printf("                           -r(estore) backupdir\n");
       printf("                           -i(nstall) filename \n");
+      printf("                           -m(erge)   filename \n");
       printf("                           -f(etch)   dbname   \n");
       printf("                           -d(elete)  dbname   \n");
       printf("                           -e(xclude) filename\n");
@@ -506,7 +541,7 @@ int main(int argc, char *argv[])
   device = argv[1];
   
   optind = 2;
-  while ((c = getopt(argc, argv, "b:e:r:i:f:d:plh")) != -1) {
+  while ((c = getopt(argc, argv, "b:e:r:i:m:f:d:plh")) != -1) {
     switch (c) {
     case 'b':
       Backup(optarg);
@@ -517,6 +552,9 @@ int main(int argc, char *argv[])
     case 'i':
       Install(optarg);
       break;
+    case 'm':
+      Merge(optarg);
+      break;
     case 'f':
       Fetch(optarg);
       break;
