@@ -71,11 +71,10 @@ int main(int argc, char *argv[])
     exit(1);
   }
     
-  addr.sa_family = PI_AF_SLP;
-  addr.port = 3;
-  strcpy(addr.device,argv[1]);
+  addr.pi_family = PI_AF_SLP;
+  strcpy(addr.pi_device,argv[1]);
   
-  ret = pi_bind(sd, &addr, sizeof(addr));
+  ret = pi_bind(sd, (struct sockaddr*)&addr, sizeof(addr));
   if(ret == -1) {
     perror("pi_bind");
     exit(1);
@@ -118,8 +117,10 @@ int main(int argc, char *argv[])
   for (i=0;1;i++) {
   	struct ToDo t;
   	int attr, category;
+  	recordid_t id;
+  	char id_buf[255];
   	                           
-  	int len = dlp_ReadRecordByIndex(sd, db, i, buffer, 0, 0, &attr, &category);
+  	int len = dlp_ReadRecordByIndex(sd, db, i, buffer, &id, 0, &attr, &category);
   	if(len<0)
   		break;
   		
@@ -134,6 +135,8 @@ int main(int argc, char *argv[])
 	fprintf(ical,"$n date [date today]\n");
 	fprintf(ical,"$n todo 1\n");
 	fprintf(ical,"$n option Priority %d\n", t.priority);
+        sprintf(id_buf, "%lx", id);
+        fprintf(ical,"$i option PilotRecordId %s\n", id_buf);
 	fprintf(ical,"$n done %d\n", t.complete ? 1 : 0);
 	fprintf(ical,"cal add $n\n");
 	
@@ -159,8 +162,10 @@ int main(int argc, char *argv[])
   	int j;
   	struct Appointment a;
   	int attr;
+  	recordid_t id;
+  	char id_buf[255];
   	                           
-  	int len = dlp_ReadRecordByIndex(sd, db, i, buffer, 0, 0, &attr, 0);
+  	int len = dlp_ReadRecordByIndex(sd, db, i, buffer, &id, 0, &attr, 0);
   	if(len<0)
   		break;
   		
@@ -253,6 +258,9 @@ int main(int argc, char *argv[])
 	} else 
           fprintf(ical,"$i date $begin\n");
 	
+        sprintf(id_buf, "%lx", id);
+        fprintf(ical,"$i option PilotRecordId %s\n", id_buf);
+        
 	fprintf(ical,"cal add $i\n");
 
 	free_Appointment(&a);
