@@ -754,6 +754,8 @@ int main(int argc, char *argv[])
   abs.AppendArchive = AppendArchive;
 #endif  
 
+  fprintf(stderr, "A Warning be upon Ye: Here Be Dragons!\nThis program is incomplete, ill-considered, and unreliable!\nDo not Rely on without Consideration of Completing the program...\n\n");
+
   if (argc < 3) {
     fprintf(stderr,"usage:%s %s calfile # Calfile will be overwritten!\n",argv[0],TTYPrompt);
     exit(2);
@@ -787,11 +789,10 @@ int main(int argc, char *argv[])
   abs.ical = &ical;
   abs.cal = "calmain";
   
-  addr.sa_family = PI_AF_SLP;
-  addr.port = 3;
-  strcpy(addr.device,argv[1]);
+  addr.pi_family = PI_AF_SLP;
+  strcpy(addr.pi_device,argv[1]);
   
-  ret = pi_bind(sd, &addr, sizeof(addr));
+  ret = pi_bind(sd, (struct sockaddr*)&addr, sizeof(addr));
   if(ret == -1) {
     perror("pi_bind");
     exit(1);
